@@ -17,22 +17,22 @@
  *
  **********************************************************************/
 
-#include <locale>     // for std::locale::classic
-#include <memory>     // for std::unique_ptr
-#include <sstream>    // for std::stringstream
-#include "baseapi.h"  // for TessBaseAPI
+#include <tesseract/baseapi.h> // for TessBaseAPI
+#include <locale>              // for std::locale::classic
+#include <memory>              // for std::unique_ptr
+#include <sstream>             // for std::stringstream
 #ifdef _WIN32
-# include "host.h"    // windows.h for MultiByteToWideChar, ...
+#  include "host.h" // windows.h for MultiByteToWideChar, ...
 #endif
-#include "renderer.h"
-#include "tesseractclass.h"  // for Tesseract
+#include <tesseract/renderer.h>
+#include "tesseractclass.h" // for Tesseract
 
 namespace tesseract {
 
 /**
  * Gets the block orientation at the current iterator position.
  */
-static tesseract::Orientation GetBlockTextOrientation(const PageIterator* it) {
+static tesseract::Orientation GetBlockTextOrientation(const PageIterator *it) {
   tesseract::Orientation orientation;
   tesseract::WritingDirection writing_direction;
   tesseract::TextlineOrder textline_order;
@@ -50,9 +50,9 @@ static tesseract::Orientation GetBlockTextOrientation(const PageIterator* it) {
  * method currently only inserts a 'textangle' property to indicate the rotation
  * direction and does not add any baseline information to the hocr string.
  */
-static void AddBaselineCoordsTohOCR(const PageIterator* it,
+static void AddBaselineCoordsTohOCR(const PageIterator *it,
                                     PageIteratorLevel level,
-                                    std::stringstream& hocr_str) {
+                                    std::stringstream &hocr_str) {
   tesseract::Orientation orientation = GetBlockTextOrientation(it);
   if (orientation != ORIENTATION_PAGE_UP) {
     hocr_str << "; textangle " << 360 - orientation * 90;
@@ -64,7 +64,9 @@ static void AddBaselineCoordsTohOCR(const PageIterator* it,
 
   // Try to get the baseline coordinates at this level.
   int x1, y1, x2, y2;
-  if (!it->Baseline(level, &x1, &y1, &x2, &y2)) return;
+  if (!it->Baseline(level, &x1, &y1, &x2, &y2)) {
+    return;
+  }
   // Following the description of this field of the hOCR spec, we convert the
   // baseline coordinates so that "the bottom left of the bounding box is the
   // origin".
@@ -86,8 +88,8 @@ static void AddBaselineCoordsTohOCR(const PageIterator* it,
            << round(p0 * 1000.0) / 1000.0;
 }
 
-static void AddBoxTohOCR(const ResultIterator* it, PageIteratorLevel level,
-                         std::stringstream& hocr_str) {
+static void AddBoxTohOCR(const ResultIterator *it, PageIteratorLevel level,
+                         std::stringstream &hocr_str) {
   int left, top, right, bottom;
   it->BoundingBox(level, &left, &top, &right, &bottom);
   // This is the only place we use double quotes instead of single quotes,
@@ -98,7 +100,7 @@ static void AddBoxTohOCR(const ResultIterator* it, PageIteratorLevel level,
   if (level == RIL_TEXTLINE) {
     AddBaselineCoordsTohOCR(it, level, hocr_str);
     // add custom height measures
-    float row_height, descenders, ascenders;  // row attributes
+    float row_height, descenders, ascenders; // row attributes
     it->RowAttributes(&row_height, &descenders, &ascenders);
     // TODO(rays): Do we want to limit these to a single decimal place?
     hocr_str << "; x_size " << row_height << "; x_descenders " << -descenders
@@ -116,7 +118,7 @@ static void AddBoxTohOCR(const ResultIterator* it, PageIteratorLevel level,
  * STL removed from original patch submission and refactored by rays.
  * Returned string must be freed with the delete [] operator.
  */
-char* TessBaseAPI::GetHOCRText(int page_number) {
+char *TessBaseAPI::GetHOCRText(int page_number) {
   return GetHOCRText(nullptr, page_number);
 }
 
@@ -129,34 +131,38 @@ char* TessBaseAPI::GetHOCRText(int page_number) {
  * STL removed from original patch submission and refactored by rays.
  * Returned string must be freed with the delete [] operator.
  */
-char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
-  if (tesseract_ == nullptr || (page_res_ == nullptr && Recognize(monitor) < 0))
+char *TessBaseAPI::GetHOCRText(ETEXT_DESC *monitor, int page_number) {
+  if (tesseract_ == nullptr ||
+      (page_res_ == nullptr && Recognize(monitor) < 0)) {
     return nullptr;
+  }
 
-  int lcnt = 1, bcnt = 1, pcnt = 1, wcnt = 1, scnt = 1, tcnt = 1, gcnt = 1;
-  int page_id = page_number + 1;  // hOCR uses 1-based page numbers.
-  bool para_is_ltr = true;        // Default direction is LTR
-  const char* paragraph_lang = nullptr;
+  int lcnt = 1, bcnt = 1, pcnt = 1, wcnt = 1, scnt = 1, tcnt = 1, ccnt = 1;
+  int page_id = page_number + 1; // hOCR uses 1-based page numbers.
+  bool para_is_ltr = true;       // Default direction is LTR
+  const char *paragraph_lang = nullptr;
   bool font_info = false;
   bool hocr_boxes = false;
   GetBoolVariable("hocr_font_info", &font_info);
   GetBoolVariable("hocr_char_boxes", &hocr_boxes);
 
-  if (input_file_ == nullptr) SetInputName(nullptr);
+  if (input_file_.empty()) {
+    SetInputName(nullptr);
+  }
 
 #ifdef _WIN32
   // convert input name from ANSI encoding to utf-8
   int str16_len =
-      MultiByteToWideChar(CP_ACP, 0, input_file_->string(), -1, nullptr, 0);
-  wchar_t* uni16_str = new WCHAR[str16_len];
-  str16_len = MultiByteToWideChar(CP_ACP, 0, input_file_->string(), -1,
-                                  uni16_str, str16_len);
+      MultiByteToWideChar(CP_ACP, 0, input_file_.c_str(), -1, nullptr, 0);
+  wchar_t *uni16_str = new WCHAR[str16_len];
+  str16_len = MultiByteToWideChar(CP_ACP, 0, input_file_.c_str(), -1, uni16_str,
+                                  str16_len);
   int utf8_len = WideCharToMultiByte(CP_UTF8, 0, uni16_str, str16_len, nullptr,
                                      0, nullptr, nullptr);
-  char* utf8_str = new char[utf8_len];
+  char *utf8_str = new char[utf8_len];
   WideCharToMultiByte(CP_UTF8, 0, uni16_str, str16_len, utf8_str, utf8_len,
                       nullptr, nullptr);
-  *input_file_ = utf8_str;
+  input_file_ = utf8_str;
   delete[] uni16_str;
   delete[] utf8_str;
 #endif
@@ -166,18 +172,20 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
   hocr_str.imbue(std::locale::classic());
   // Use 8 digits for double values.
   hocr_str.precision(8);
-  hocr_str << "  <div class='ocr_page'";
-  hocr_str << " id='"
-           << "page_" << page_id << "'";
-  hocr_str << " title='image \"";
-  if (input_file_) {
-    hocr_str << HOcrEscape(input_file_->string()).c_str();
+  hocr_str << "  <div class='ocr_page'"
+           << " id='"
+           << "page_" << page_id << "'"
+           << " title='image \"";
+  if (!input_file_.empty()) {
+    hocr_str << HOcrEscape(input_file_.c_str());
   } else {
     hocr_str << "unknown";
   }
+
   hocr_str << "\"; bbox " << rect_left_ << " " << rect_top_ << " "
            << rect_width_ << " " << rect_height_ << "; ppageno " << page_number
-           << "'>\n";
+           << "; scan_res " << GetSourceYResolution() << " "
+           << GetSourceYResolution() << "'>\n";
 
   std::unique_ptr<ResultIterator> res_it(GetIterator());
   while (!res_it->Empty(RIL_BLOCK)) {
@@ -188,7 +196,7 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
 
     // Open any new block/paragraph/textline.
     if (res_it->IsAtBeginningOf(RIL_BLOCK)) {
-      para_is_ltr = true;  // reset to default direction
+      para_is_ltr = true; // reset to default direction
       hocr_str << "   <div class='ocr_carea'"
                << " id='"
                << "block_" << page_id << "_" << bcnt << "'";
@@ -229,11 +237,13 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
     }
 
     // Now, process the word...
-    std::vector<std::vector<std::pair<const char*, float>>>* choiceMap =
-        nullptr;
-    if (tesseract_->lstm_choice_mode) {
-
-      choiceMap = res_it->GetBestLSTMSymbolChoices();
+    int32_t lstm_choice_mode = tesseract_->lstm_choice_mode;
+    std::vector<std::vector<std::vector<std::pair<const char *, float>>>>
+        *rawTimestepMap = nullptr;
+    std::vector<std::vector<std::pair<const char *, float>>> *CTCMap = nullptr;
+    if (lstm_choice_mode) {
+      CTCMap = res_it->GetBestLSTMSymbolChoices();
+      rawTimestepMap = res_it->GetRawLSTMTimesteps();
     }
     hocr_str << "\n      <span class='ocrx_word'"
              << " id='"
@@ -241,7 +251,7 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
     int left, top, right, bottom;
     bool bold, italic, underlined, monospace, serif, smallcaps;
     int pointsize, font_id;
-    const char* font_name;
+    const char *font_name;
     res_it->BoundingBox(RIL_WORD, &left, &top, &right, &bottom);
     font_name =
         res_it->WordFontAttributes(&bold, &italic, &underlined, &monospace,
@@ -256,90 +266,169 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
       hocr_str << "; x_fsize " << pointsize;
     }
     hocr_str << "'";
-    const char* lang = res_it->WordRecognitionLanguage();
+    const char *lang = res_it->WordRecognitionLanguage();
     if (lang && (!paragraph_lang || strcmp(lang, paragraph_lang))) {
       hocr_str << " lang='" << lang << "'";
     }
     switch (res_it->WordDirection()) {
       // Only emit direction if different from current paragraph direction
       case DIR_LEFT_TO_RIGHT:
-        if (!para_is_ltr) hocr_str << " dir='ltr'";
+        if (!para_is_ltr) {
+          hocr_str << " dir='ltr'";
+        }
         break;
       case DIR_RIGHT_TO_LEFT:
-        if (para_is_ltr) hocr_str << " dir='rtl'";
+        if (para_is_ltr) {
+          hocr_str << " dir='rtl'";
+        }
         break;
       case DIR_MIX:
       case DIR_NEUTRAL:
-      default:  // Do nothing.
+      default: // Do nothing.
         break;
     }
     hocr_str << ">";
     bool last_word_in_line = res_it->IsAtFinalElement(RIL_TEXTLINE, RIL_WORD);
     bool last_word_in_para = res_it->IsAtFinalElement(RIL_PARA, RIL_WORD);
     bool last_word_in_block = res_it->IsAtFinalElement(RIL_BLOCK, RIL_WORD);
-    if (bold) hocr_str << "<strong>";
-    if (italic) hocr_str << "<em>";
+    if (bold) {
+      hocr_str << "<strong>";
+    }
+    if (italic) {
+      hocr_str << "<em>";
+    }
     do {
       const std::unique_ptr<const char[]> grapheme(
           res_it->GetUTF8Text(RIL_SYMBOL));
       if (grapheme && grapheme[0] != 0) {
         if (hocr_boxes) {
           res_it->BoundingBox(RIL_SYMBOL, &left, &top, &right, &bottom);
-          hocr_str << "\n             <span class='ocrx_cinfo' title='x_bboxes "
+          hocr_str << "\n       <span class='ocrx_cinfo' title='x_bboxes "
                    << left << " " << top << " " << right << " " << bottom
                    << "; x_conf " << res_it->Confidence(RIL_SYMBOL) << "'>";
         }
         hocr_str << HOcrEscape(grapheme.get()).c_str();
         if (hocr_boxes) {
           hocr_str << "</span>";
+          tesseract::ChoiceIterator ci(*res_it);
+          if (lstm_choice_mode == 1 && ci.Timesteps() != nullptr) {
+            std::vector<std::vector<std::pair<const char *, float>>> *symbol =
+                ci.Timesteps();
+            hocr_str << "\n        <span class='ocr_symbol'"
+                     << " id='"
+                     << "symbol_" << page_id << "_" << wcnt << "_" << scnt
+                     << "'>";
+            for (const auto &timestep : *symbol) {
+              hocr_str << "\n         <span class='ocrx_cinfo'"
+                       << " id='"
+                       << "timestep" << page_id << "_" << wcnt << "_" << tcnt
+                       << "'>";
+              for (auto conf : timestep) {
+                hocr_str << "\n          <span class='ocrx_cinfo'"
+                         << " id='"
+                         << "choice_" << page_id << "_" << wcnt << "_" << ccnt
+                         << "'"
+                         << " title='x_confs " << int(conf.second * 100) << "'>"
+                         << HOcrEscape(conf.first).c_str() << "</span>";
+                ++ccnt;
+              }
+              hocr_str << "</span>";
+              ++tcnt;
+            }
+            hocr_str << "\n        </span>";
+            ++scnt;
+          } else if (lstm_choice_mode == 2) {
+            hocr_str << "\n        <span class='ocrx_cinfo'"
+                     << " id='"
+                     << "lstm_choices_" << page_id << "_" << wcnt << "_" << tcnt
+                     << "'>";
+            do {
+              const char *choice = ci.GetUTF8Text();
+              float choiceconf = ci.Confidence();
+              if (choice != nullptr) {
+                hocr_str << "\n         <span class='ocrx_cinfo'"
+                         << " id='"
+                         << "choice_" << page_id << "_" << wcnt << "_" << ccnt
+                         << "'"
+                         << " title='x_confs " << choiceconf << "'>"
+                         << HOcrEscape(choice).c_str() << "</span>";
+                ccnt++;
+              }
+            } while (ci.Next());
+            hocr_str << "\n        </span>";
+            tcnt++;
+          }
         }
       }
       res_it->Next(RIL_SYMBOL);
     } while (!res_it->Empty(RIL_BLOCK) && !res_it->IsAtBeginningOf(RIL_WORD));
-    if (italic) hocr_str << "</em>";
-    if (bold) hocr_str << "</strong>";
+    if (italic) {
+      hocr_str << "</em>";
+    }
+    if (bold) {
+      hocr_str << "</strong>";
+    }
     // If the lstm choice mode is required it is added here
-    if (tesseract_->lstm_choice_mode == 1 && choiceMap != nullptr) {
-      for (auto timestep : *choiceMap) {
-        hocr_str << "\n       <span class='ocrx_cinfo'"
+    if (lstm_choice_mode == 1 && !hocr_boxes && rawTimestepMap != nullptr) {
+      for (const auto &symbol : *rawTimestepMap) {
+        hocr_str << "\n       <span class='ocr_symbol'"
                  << " id='"
-                 << "timestep_" << page_id << "_" << wcnt << "_" << tcnt << "'"
-                 << ">";
-        for (std::pair<const char*, float> conf : timestep) {
-          hocr_str << "<span class='ocr_glyph'"
+                 << "symbol_" << page_id << "_" << wcnt << "_" << scnt << "'>";
+        for (const auto &timestep : symbol) {
+          hocr_str << "\n        <span class='ocrx_cinfo'"
                    << " id='"
-                   << "choice_" << page_id << "_" << wcnt << "_" << gcnt << "'"
-                   << " title='x_confs " << int(conf.second * 100) << "'>"
-                   << conf.first << "</span>";
-          gcnt++;
+                   << "timestep" << page_id << "_" << wcnt << "_" << tcnt
+                   << "'>";
+          for (auto conf : timestep) {
+            hocr_str << "\n         <span class='ocrx_cinfo'"
+                     << " id='"
+                     << "choice_" << page_id << "_" << wcnt << "_" << ccnt
+                     << "'"
+                     << " title='x_confs " << int(conf.second * 100) << "'>"
+                     << HOcrEscape(conf.first).c_str() << "</span>";
+            ++ccnt;
+          }
+          hocr_str << "</span>";
+          ++tcnt;
         }
         hocr_str << "</span>";
-        tcnt++;
+        ++scnt;
       }
-    } else if (tesseract_->lstm_choice_mode == 2 && choiceMap != nullptr) {
-      for (auto timestep : *choiceMap) {
+    } else if (lstm_choice_mode == 2 && !hocr_boxes && CTCMap != nullptr) {
+      for (const auto &timestep : *CTCMap) {
         if (timestep.size() > 0) {
           hocr_str << "\n       <span class='ocrx_cinfo'"
                    << " id='"
                    << "lstm_choices_" << page_id << "_" << wcnt << "_" << tcnt
                    << "'>";
-          for (auto & j : timestep) {
-            hocr_str << "<span class='ocr_glyph'"
+          for (auto &j : timestep) {
+            float conf = 100 - tesseract_->lstm_rating_coefficient * j.second;
+            if (conf < 0.0f) {
+              conf = 0.0f;
+            }
+            if (conf > 100.0f) {
+              conf = 100.0f;
+            }
+            hocr_str << "\n        <span class='ocrx_cinfo'"
                      << " id='"
-                     << "choice_" << page_id << "_" << wcnt << "_" << gcnt
+                     << "choice_" << page_id << "_" << wcnt << "_" << ccnt
                      << "'"
-                     << " title='x_confs " << int(j.second * 100)
-                     << "'>" << j.first << "</span>";
-            gcnt++;
+                     << " title='x_confs " << conf << "'>"
+                     << HOcrEscape(j.first).c_str() << "</span>";
+            ccnt++;
           }
           hocr_str << "</span>";
           tcnt++;
         }
       }
     }
+    // Close ocrx_word.
+    if (hocr_boxes || lstm_choice_mode > 0) {
+      hocr_str << "\n      ";
+    }
     hocr_str << "</span>";
     tcnt = 1;
-    gcnt = 1;
+    ccnt = 1;
     wcnt++;
     // Close any ending block/paragraph/textline.
     if (last_word_in_line) {
@@ -349,7 +438,7 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
     if (last_word_in_para) {
       hocr_str << "\n    </p>\n";
       pcnt++;
-      para_is_ltr = true;  // back to default direction
+      para_is_ltr = true; // back to default direction
     }
     if (last_word_in_block) {
       hocr_str << "   </div>\n";
@@ -358,8 +447,8 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
   }
   hocr_str << "  </div>\n";
 
-  const std::string& text = hocr_str.str();
-  char* result = new char[text.length() + 1];
+  const std::string &text = hocr_str.str();
+  char *result = new char[text.length() + 1];
   strcpy(result, text.c_str());
   return result;
 }
@@ -367,12 +456,12 @@ char* TessBaseAPI::GetHOCRText(ETEXT_DESC* monitor, int page_number) {
 /**********************************************************************
  * HOcr Text Renderer interface implementation
  **********************************************************************/
-TessHOcrRenderer::TessHOcrRenderer(const char* outputbase)
+TessHOcrRenderer::TessHOcrRenderer(const char *outputbase)
     : TessResultRenderer(outputbase, "hocr") {
   font_info_ = false;
 }
 
-TessHOcrRenderer::TessHOcrRenderer(const char* outputbase, bool font_info)
+TessHOcrRenderer::TessHOcrRenderer(const char *outputbase, bool font_info)
     : TessResultRenderer(outputbase, "hocr") {
   font_info_ = font_info;
 }
@@ -389,11 +478,13 @@ bool TessHOcrRenderer::BeginDocumentHandler() {
       "</title>\n"
       "  <meta http-equiv=\"Content-Type\" content=\"text/html;"
       "charset=utf-8\"/>\n"
-      "  <meta name='ocr-system' content='tesseract " PACKAGE_VERSION
+      "  <meta name='ocr-system' content='tesseract " TESSERACT_VERSION_STR
       "' />\n"
       "  <meta name='ocr-capabilities' content='ocr_page ocr_carea ocr_par"
       " ocr_line ocrx_word ocrp_wconf");
-  if (font_info_) AppendString(" ocrp_lang ocrp_dir ocrp_font ocrp_fsize");
+  if (font_info_) {
+    AppendString(" ocrp_lang ocrp_dir ocrp_font ocrp_fsize");
+  }
   AppendString(
       "'/>\n"
       " </head>\n"
@@ -408,13 +499,15 @@ bool TessHOcrRenderer::EndDocumentHandler() {
   return true;
 }
 
-bool TessHOcrRenderer::AddImageHandler(TessBaseAPI* api) {
+bool TessHOcrRenderer::AddImageHandler(TessBaseAPI *api) {
   const std::unique_ptr<const char[]> hocr(api->GetHOCRText(imagenum()));
-  if (hocr == nullptr) return false;
+  if (hocr == nullptr) {
+    return false;
+  }
 
   AppendString(hocr.get());
 
   return true;
 }
 
-}  // namespace tesseract
+} // namespace tesseract
