@@ -3,7 +3,6 @@
 // Description: Iterator for tesseract page structure that avoids using
 //              tesseract internal data structures.
 // Author:      Ray Smith
-// Created:     Fri Feb 26 14:32:09 PST 2010
 //
 // (C) Copyright 2010, Google Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,8 +17,8 @@
 //
 ///////////////////////////////////////////////////////////////////////
 
-#include "pageiterator.h"
-#include "allheaders.h"
+#include <allheaders.h>
+#include <tesseract/pageiterator.h>
 #include "helpers.h"
 #include "pageres.h"
 #include "tesseractclass.h"
@@ -28,7 +27,7 @@
 
 namespace tesseract {
 
-PageIterator::PageIterator(PAGE_RES* page_res, Tesseract* tesseract, int scale,
+PageIterator::PageIterator(PAGE_RES *page_res, Tesseract *tesseract, int scale,
                            int scaled_yres, int rect_left, int rect_top,
                            int rect_width, int rect_height)
     : page_res_(page_res),
@@ -59,7 +58,7 @@ PageIterator::~PageIterator() {
  * all the objects at a lower level, while maintaining an iterator to
  * objects at a higher level.
  */
-PageIterator::PageIterator(const PageIterator& src)
+PageIterator::PageIterator(const PageIterator &src)
     : page_res_(src.page_res_),
       tesseract_(src.tesseract_),
       word_(nullptr),
@@ -78,7 +77,7 @@ PageIterator::PageIterator(const PageIterator& src)
   BeginWord(src.blob_index_);
 }
 
-const PageIterator& PageIterator::operator=(const PageIterator& src) {
+const PageIterator &PageIterator::operator=(const PageIterator &src) {
   page_res_ = src.page_res_;
   tesseract_ = src.tesseract_;
   include_upper_dots_ = src.include_upper_dots_;
@@ -95,9 +94,9 @@ const PageIterator& PageIterator::operator=(const PageIterator& src) {
   return *this;
 }
 
-bool PageIterator::PositionedAtSameWord(const PAGE_RES_IT* other) const {
+bool PageIterator::PositionedAtSameWord(const PAGE_RES_IT *other) const {
   return (it_ == nullptr && it_ == other) ||
-     ((other != nullptr) && (it_ != nullptr) && (*it_ == *other));
+         ((other != nullptr) && (it_ != nullptr) && (*it_ == *other));
 }
 
 // ============= Moving around within the page ============.
@@ -109,7 +108,9 @@ void PageIterator::Begin() {
 }
 
 void PageIterator::RestartParagraph() {
-  if (it_->block() == nullptr) return; // At end of the document.
+  if (it_->block() == nullptr) {
+    return; // At end of the document.
+  }
   PAGE_RES_IT para(page_res_);
   PAGE_RES_IT next_para(para);
   next_para.forward_paragraph();
@@ -146,9 +147,12 @@ void PageIterator::RestartRow() {
  * the appropriate language has been loaded into Tesseract.
  */
 bool PageIterator::Next(PageIteratorLevel level) {
-  if (it_->block() == nullptr) return false;  // Already at the end!
-  if (it_->word() == nullptr)
+  if (it_->block() == nullptr) {
+    return false; // Already at the end!
+  }
+  if (it_->word() == nullptr) {
     level = RIL_BLOCK;
+  }
 
   switch (level) {
     case RIL_BLOCK:
@@ -159,19 +163,23 @@ bool PageIterator::Next(PageIteratorLevel level) {
       break;
     case RIL_TEXTLINE:
       for (it_->forward_with_empties(); it_->row() == it_->prev_row();
-           it_->forward_with_empties());
+           it_->forward_with_empties()) {
+        ;
+      }
       break;
     case RIL_WORD:
       it_->forward_with_empties();
       break;
     case RIL_SYMBOL:
-      if (cblob_it_ != nullptr)
+      if (cblob_it_ != nullptr) {
         cblob_it_->forward();
+      }
       ++blob_index_;
-      if (blob_index_ >= word_length_)
+      if (blob_index_ >= word_length_) {
         it_->forward_with_empties();
-      else
+      } else {
         return true;
+      }
       break;
   }
   BeginWord(0);
@@ -184,15 +192,19 @@ bool PageIterator::Next(PageIteratorLevel level) {
  * moved to the start of a RIL_PARA.
  */
 bool PageIterator::IsAtBeginningOf(PageIteratorLevel level) const {
-  if (it_->block() == nullptr) return false;  // Already at the end!
-  if (it_->word() == nullptr) return true;  // In an image block.
+  if (it_->block() == nullptr) {
+    return false; // Already at the end!
+  }
+  if (it_->word() == nullptr) {
+    return true; // In an image block.
+  }
   switch (level) {
     case RIL_BLOCK:
       return blob_index_ == 0 && it_->block() != it_->prev_block();
     case RIL_PARA:
       return blob_index_ == 0 &&
-          (it_->block() != it_->prev_block() ||
-           it_->row()->row->para() != it_->prev_row()->row->para());
+             (it_->block() != it_->prev_block() ||
+              it_->row()->row->para() != it_->prev_row()->row->para());
     case RIL_TEXTLINE:
       return blob_index_ == 0 && it_->row() != it_->prev_row();
     case RIL_WORD:
@@ -209,7 +221,9 @@ bool PageIterator::IsAtBeginningOf(PageIteratorLevel level) const {
  */
 bool PageIterator::IsAtFinalElement(PageIteratorLevel level,
                                     PageIteratorLevel element) const {
-  if (Empty(element)) return true;  // Already at the end!
+  if (Empty(element)) {
+    return true; // Already at the end!
+  }
   // The result is true if we step forward by element and find we are
   // at the the end of the page or at beginning of *all* levels in:
   // [level, element).
@@ -218,11 +232,14 @@ bool PageIterator::IsAtFinalElement(PageIteratorLevel level,
   // word on a line, so we also have to be at the first symbol in a word.
   PageIterator next(*this);
   next.Next(element);
-  if (next.Empty(element)) return true;  // Reached the end of the page.
+  if (next.Empty(element)) {
+    return true; // Reached the end of the page.
+  }
   while (element > level) {
     element = static_cast<PageIteratorLevel>(element - 1);
-    if (!next.IsAtBeginningOf(element))
+    if (!next.IsAtBeginningOf(element)) {
       return false;
+    }
   }
   return true;
 }
@@ -235,12 +252,15 @@ bool PageIterator::IsAtFinalElement(PageIteratorLevel level,
  */
 int PageIterator::Cmp(const PageIterator &other) const {
   int word_cmp = it_->cmp(*other.it_);
-  if (word_cmp != 0)
+  if (word_cmp != 0) {
     return word_cmp;
-  if (blob_index_ < other.blob_index_)
+  }
+  if (blob_index_ < other.blob_index_) {
     return -1;
-  if (blob_index_ == other.blob_index_)
+  }
+  if (blob_index_ == other.blob_index_) {
     return 0;
+  }
   return 1;
 }
 
@@ -263,11 +283,12 @@ int PageIterator::Cmp(const PageIterator &other) const {
  * See comment on coordinate system above.
  * Returns false if there is no such object at the current position.
  */
-bool PageIterator::BoundingBoxInternal(PageIteratorLevel level,
-                                       int* left, int* top,
-                                       int* right, int* bottom) const {
-  if (Empty(level))
+bool PageIterator::BoundingBoxInternal(PageIteratorLevel level, int *left,
+                                       int *top, int *right,
+                                       int *bottom) const {
+  if (Empty(level)) {
     return false;
+  }
   TBOX box;
   PARA *para = nullptr;
   switch (level) {
@@ -287,10 +308,11 @@ bool PageIterator::BoundingBoxInternal(PageIteratorLevel level,
                                                        include_lower_dots_);
       break;
     case RIL_SYMBOL:
-      if (cblob_it_ == nullptr)
+      if (cblob_it_ == nullptr) {
         box = it_->word()->box_word->BlobBox(blob_index_);
-      else
+      } else {
         box = cblob_it_->data()->bounding_box();
+      }
   }
   if (level == RIL_PARA) {
     PageIterator other = *this;
@@ -304,8 +326,9 @@ bool PageIterator::BoundingBoxInternal(PageIteratorLevel level,
       }
     } while (other.Next(RIL_TEXTLINE));
   }
-  if (level != RIL_SYMBOL || cblob_it_ != nullptr)
+  if (level != RIL_SYMBOL || cblob_it_ != nullptr) {
     box.rotate(it_->block()->block->re_rotation());
+  }
   // Now we have a box in tesseract coordinates relative to the image rectangle,
   // we have to convert the coords to a top-down system.
   const int pix_height = pixGetHeight(tesseract_->pix_binary());
@@ -323,22 +346,22 @@ bool PageIterator::BoundingBoxInternal(PageIteratorLevel level,
  * See comment on coordinate system above.
  * Returns false if there is no such object at the current position.
  */
-bool PageIterator::BoundingBox(PageIteratorLevel level,
-                               int* left, int* top,
-                               int* right, int* bottom) const {
+bool PageIterator::BoundingBox(PageIteratorLevel level, int *left, int *top,
+                               int *right, int *bottom) const {
   return BoundingBox(level, 0, left, top, right, bottom);
 }
 
 bool PageIterator::BoundingBox(PageIteratorLevel level, const int padding,
-                               int* left, int* top,
-                               int* right, int* bottom) const {
-  if (!BoundingBoxInternal(level, left, top, right, bottom))
+                               int *left, int *top, int *right,
+                               int *bottom) const {
+  if (!BoundingBoxInternal(level, left, top, right, bottom)) {
     return false;
+  }
   // Convert to the coordinate system of the original image.
-  *left = ClipToRange(*left / scale_ + rect_left_ - padding,
-                      rect_left_, rect_left_ + rect_width_);
-  *top = ClipToRange(*top / scale_ + rect_top_ - padding,
-                     rect_top_, rect_top_ + rect_height_);
+  *left = ClipToRange(*left / scale_ + rect_left_ - padding, rect_left_,
+                      rect_left_ + rect_width_);
+  *top = ClipToRange(*top / scale_ + rect_top_ - padding, rect_top_,
+                     rect_top_ + rect_height_);
   *right = ClipToRange((*right + scale_ - 1) / scale_ + rect_left_ + padding,
                        *left, rect_left_ + rect_width_);
   *bottom = ClipToRange((*bottom + scale_ - 1) / scale_ + rect_top_ + padding,
@@ -348,37 +371,55 @@ bool PageIterator::BoundingBox(PageIteratorLevel level, const int padding,
 
 /** Return that there is no such object at a given level. */
 bool PageIterator::Empty(PageIteratorLevel level) const {
-  if (it_->block() == nullptr) return true;  // Already at the end!
-  if (it_->word() == nullptr && level != RIL_BLOCK) return true;  // image block
-  if (level == RIL_SYMBOL && blob_index_ >= word_length_)
-    return true;  // Zero length word, or already at the end of it.
+  if (it_->block() == nullptr) {
+    return true; // Already at the end!
+  }
+  if (it_->word() == nullptr && level != RIL_BLOCK) {
+    return true; // image block
+  }
+  if (level == RIL_SYMBOL && blob_index_ >= word_length_) {
+    return true; // Zero length word, or already at the end of it.
+  }
   return false;
 }
 
-/** Returns the type of the current block. See apitypes.h for PolyBlockType. */
+/** Returns the type of the current block.
+ *  See tesseract/publictypes.h for PolyBlockType. */
 PolyBlockType PageIterator::BlockType() const {
-  if (it_->block() == nullptr || it_->block()->block == nullptr)
-    return PT_UNKNOWN;  // Already at the end!
-  if (it_->block()->block->pdblk.poly_block() == nullptr)
-    return PT_FLOWING_TEXT;  // No layout analysis used - assume text.
+  if (it_->block() == nullptr || it_->block()->block == nullptr) {
+    return PT_UNKNOWN; // Already at the end!
+  }
+  if (it_->block()->block->pdblk.poly_block() == nullptr) {
+    return PT_FLOWING_TEXT; // No layout analysis used - assume text.
+  }
   return it_->block()->block->pdblk.poly_block()->isA();
 }
 
 /** Returns the polygon outline of the current block. The returned Pta must
  *  be ptaDestroy-ed after use. */
-Pta* PageIterator::BlockPolygon() const {
-  if (it_->block() == nullptr || it_->block()->block == nullptr)
-    return nullptr;  // Already at the end!
-  if (it_->block()->block->pdblk.poly_block() == nullptr)
-    return nullptr;  // No layout analysis used - no polygon.
-  ICOORDELT_IT it(it_->block()->block->pdblk.poly_block()->points());
-  Pta* pta = ptaCreate(it.length());
+Pta *PageIterator::BlockPolygon() const {
+  if (it_->block() == nullptr || it_->block()->block == nullptr) {
+    return nullptr; // Already at the end!
+  }
+  if (it_->block()->block->pdblk.poly_block() == nullptr) {
+    return nullptr; // No layout analysis used - no polygon.
+  }
+  // Copy polygon, so we can unrotate it to image coordinates.
+  POLY_BLOCK *internal_poly = it_->block()->block->pdblk.poly_block();
+  ICOORDELT_LIST vertices;
+  vertices.deep_copy(internal_poly->points(), ICOORDELT::deep_copy);
+  POLY_BLOCK poly(&vertices, internal_poly->isA());
+  poly.rotate(it_->block()->block->re_rotation());
+  ICOORDELT_IT it(poly.points());
+  Pta *pta = ptaCreate(it.length());
   int num_pts = 0;
   for (it.mark_cycle_pt(); !it.cycled_list(); it.forward(), ++num_pts) {
-    ICOORD* pt = it.data();
+    ICOORD *pt = it.data();
     // Convert to top-down coords within the input image.
-    float x = static_cast<float>(pt->x()) / scale_ + rect_left_;
-    float y = rect_top_ + rect_height_ - static_cast<float>(pt->y()) / scale_;
+    int x = static_cast<float>(pt->x()) / scale_ + rect_left_;
+    int y = rect_top_ + rect_height_ - static_cast<float>(pt->y()) / scale_;
+    x = ClipToRange(x, rect_left_, rect_left_ + rect_width_);
+    y = ClipToRange(y, rect_top_, rect_top_ + rect_height_);
     ptaAddPt(pta, x, y);
   }
   return pta;
@@ -406,27 +447,29 @@ Pta* PageIterator::BlockPolygon() const {
  * should be good, even with xor, since the images come from the connected
  * components.
  */
-Pix* PageIterator::GetBinaryImage(PageIteratorLevel level) const {
+Pix *PageIterator::GetBinaryImage(PageIteratorLevel level) const {
   int left, top, right, bottom;
-  if (!BoundingBoxInternal(level, &left, &top, &right, &bottom))
+  if (!BoundingBoxInternal(level, &left, &top, &right, &bottom)) {
     return nullptr;
+  }
   if (level == RIL_SYMBOL && cblob_it_ != nullptr &&
-      cblob_it_->data()->area() != 0)
+      cblob_it_->data()->area() != 0) {
     return cblob_it_->data()->render();
-  Box* box = boxCreate(left, top, right - left, bottom - top);
-  Pix* pix = pixClipRectangle(tesseract_->pix_binary(), box, nullptr);
+  }
+  Box *box = boxCreate(left, top, right - left, bottom - top);
+  Image pix = pixClipRectangle(tesseract_->pix_binary(), box, nullptr);
   boxDestroy(&box);
   if (level == RIL_BLOCK || level == RIL_PARA) {
     // Clip to the block polygon as well.
     TBOX mask_box;
-    Pix* mask = it_->block()->block->render_mask(&mask_box);
+    Image mask = it_->block()->block->render_mask(&mask_box);
     int mask_x = left - mask_box.left();
     int mask_y = top - (tesseract_->ImageHeight() - mask_box.top());
     // AND the mask and pix, putting the result in pix.
-    pixRasterop(pix, std::max(0, -mask_x), std::max(0, -mask_y), pixGetWidth(pix),
-                pixGetHeight(pix), PIX_SRC & PIX_DST, mask, std::max(0, mask_x),
-                std::max(0, mask_y));
-    pixDestroy(&mask);
+    pixRasterop(pix, std::max(0, -mask_x), std::max(0, -mask_y),
+                pixGetWidth(pix), pixGetHeight(pix), PIX_SRC & PIX_DST, mask,
+                std::max(0, mask_x), std::max(0, mask_y));
+    mask.destroy();
   }
   return pix;
 }
@@ -442,41 +485,43 @@ Pix* PageIterator::GetBinaryImage(PageIteratorLevel level) const {
  * If you do not supply an original image, you will get a binary one.
  * Use pixDestroy to delete the image after use.
  */
-Pix* PageIterator::GetImage(PageIteratorLevel level, int padding,
-                            Pix* original_img,
-                            int* left, int* top) const {
+Pix *PageIterator::GetImage(PageIteratorLevel level, int padding,
+                            Pix *original_img, int *left, int *top) const {
   int right, bottom;
-  if (!BoundingBox(level, left, top, &right, &bottom))
+  if (!BoundingBox(level, left, top, &right, &bottom)) {
     return nullptr;
-  if (original_img == nullptr)
+  }
+  if (original_img == nullptr) {
     return GetBinaryImage(level);
+  }
 
   // Expand the box.
   *left = std::max(*left - padding, 0);
   *top = std::max(*top - padding, 0);
   right = std::min(right + padding, rect_width_);
   bottom = std::min(bottom + padding, rect_height_);
-  Box* box = boxCreate(*left, *top, right - *left, bottom - *top);
-  Pix* grey_pix = pixClipRectangle(original_img, box, nullptr);
+  Box *box = boxCreate(*left, *top, right - *left, bottom - *top);
+  Image grey_pix = pixClipRectangle(original_img, box, nullptr);
   boxDestroy(&box);
   if (level == RIL_BLOCK || level == RIL_PARA) {
     // Clip to the block polygon as well.
     TBOX mask_box;
-    Pix* mask = it_->block()->block->render_mask(&mask_box);
+    Image mask = it_->block()->block->render_mask(&mask_box);
     // Copy the mask registered correctly into an image the size of grey_pix.
     int mask_x = *left - mask_box.left();
     int mask_y = *top - (pixGetHeight(original_img) - mask_box.top());
     int width = pixGetWidth(grey_pix);
     int height = pixGetHeight(grey_pix);
-    Pix* resized_mask = pixCreate(width, height, 1);
-    pixRasterop(resized_mask, std::max(0, -mask_x), std::max(0, -mask_y), width, height,
-                PIX_SRC, mask, std::max(0, mask_x), std::max(0, mask_y));
-    pixDestroy(&mask);
+    Image resized_mask = pixCreate(width, height, 1);
+    pixRasterop(resized_mask, std::max(0, -mask_x), std::max(0, -mask_y), width,
+                height, PIX_SRC, mask, std::max(0, mask_x),
+                std::max(0, mask_y));
+    mask.destroy();
     pixDilateBrick(resized_mask, resized_mask, 2 * padding + 1,
                    2 * padding + 1);
     pixInvert(resized_mask, resized_mask);
     pixSetMasked(grey_pix, resized_mask, UINT32_MAX);
-    pixDestroy(&resized_mask);
+    resized_mask.destroy();
   }
   return grey_pix;
 }
@@ -486,14 +531,15 @@ Pix* PageIterator::GetImage(PageIteratorLevel level, int padding,
  * The baseline is the line that passes through (x1, y1) and (x2, y2).
  * WARNING: with vertical text, baselines may be vertical!
  */
-bool PageIterator::Baseline(PageIteratorLevel level,
-                            int* x1, int* y1, int* x2, int* y2) const {
-  if (it_->word() == nullptr) return false;  // Already at the end!
-  ROW* row = it_->row()->row;
-  WERD* word = it_->word()->word;
-  TBOX box = (level == RIL_WORD || level == RIL_SYMBOL)
-           ? word->bounding_box()
-           : row->bounding_box();
+bool PageIterator::Baseline(PageIteratorLevel level, int *x1, int *y1, int *x2,
+                            int *y2) const {
+  if (it_->word() == nullptr) {
+    return false; // Already at the end!
+  }
+  ROW *row = it_->row()->row;
+  WERD *word = it_->word()->word;
+  TBOX box = (level == RIL_WORD || level == RIL_SYMBOL) ? word->bounding_box()
+                                                        : row->bounding_box();
   int left = box.left();
   ICOORD startpt(left, static_cast<int16_t>(row->base_line(left) + 0.5));
   int right = box.right();
@@ -508,11 +554,19 @@ bool PageIterator::Baseline(PageIteratorLevel level,
   return true;
 }
 
+void PageIterator::RowAttributes(float *row_height, float *descenders,
+                                 float *ascenders) const {
+  *row_height = it_->row()->row->x_height() + it_->row()->row->ascenders() -
+                it_->row()->row->descenders();
+  *descenders = it_->row()->row->descenders();
+  *ascenders = it_->row()->row->ascenders();
+}
+
 void PageIterator::Orientation(tesseract::Orientation *orientation,
                                tesseract::WritingDirection *writing_direction,
                                tesseract::TextlineOrder *textline_order,
                                float *deskew_angle) const {
-  BLOCK* block = it_->block()->block;
+  BLOCK *block = it_->block()->block;
 
   // Orientation
   FCOORD up_in_image(0.0, 1.0);
@@ -534,40 +588,36 @@ void PageIterator::Orientation(tesseract::Orientation *orientation,
   // Writing direction
   bool is_vertical_text = (block->classify_rotation().x() == 0.0);
   bool right_to_left = block->right_to_left();
-  *writing_direction =
-      is_vertical_text
-          ? WRITING_DIRECTION_TOP_TO_BOTTOM
-          : (right_to_left
-                ? WRITING_DIRECTION_RIGHT_TO_LEFT
-                : WRITING_DIRECTION_LEFT_TO_RIGHT);
+  *writing_direction = is_vertical_text
+                           ? WRITING_DIRECTION_TOP_TO_BOTTOM
+                           : (right_to_left ? WRITING_DIRECTION_RIGHT_TO_LEFT
+                                            : WRITING_DIRECTION_LEFT_TO_RIGHT);
 
   // Textline Order
-  const bool is_mongolian = false;  // TODO(eger): fix me
+  const bool is_mongolian = false; // TODO(eger): fix me
   *textline_order = is_vertical_text
-      ? (is_mongolian
-         ? TEXTLINE_ORDER_LEFT_TO_RIGHT
-         : TEXTLINE_ORDER_RIGHT_TO_LEFT)
-      : TEXTLINE_ORDER_TOP_TO_BOTTOM;
+                        ? (is_mongolian ? TEXTLINE_ORDER_LEFT_TO_RIGHT
+                                        : TEXTLINE_ORDER_RIGHT_TO_LEFT)
+                        : TEXTLINE_ORDER_TOP_TO_BOTTOM;
 
   // Deskew angle
-  FCOORD skew = block->skew();  // true horizontal for textlines
+  FCOORD skew = block->skew(); // true horizontal for textlines
   *deskew_angle = -skew.angle();
 }
 
 void PageIterator::ParagraphInfo(tesseract::ParagraphJustification *just,
-                                 bool *is_list_item,
-                                 bool *is_crown,
+                                 bool *is_list_item, bool *is_crown,
                                  int *first_line_indent) const {
   *just = tesseract::JUSTIFICATION_UNKNOWN;
   if (!it_->row() || !it_->row()->row || !it_->row()->row->para() ||
-      !it_->row()->row->para()->model)
+      !it_->row()->row->para()->model) {
     return;
+  }
 
   PARA *para = it_->row()->row->para();
   *is_list_item = para->is_list_item;
   *is_crown = para->is_very_first_or_continuation;
-  *first_line_indent = para->model->first_indent() -
-      para->model->body_indent();
+  *first_line_indent = para->model->first_indent() - para->model->body_indent();
   *just = para->model->justification();
 }
 
@@ -576,7 +626,7 @@ void PageIterator::ParagraphInfo(tesseract::ParagraphJustification *just,
  * moves the iterator to the given offset.
  */
 void PageIterator::BeginWord(int offset) {
-  WERD_RES* word_res = it_->word();
+  WERD_RES *word_res = it_->word();
   if (word_res == nullptr) {
     // This is a non-text block, so there is no word.
     word_length_ = 0;
@@ -589,13 +639,14 @@ void PageIterator::BeginWord(int offset) {
     // is already baseline denormalized.
     word_length_ = word_res->best_choice->length();
     if (word_res->box_word != nullptr) {
-      if (word_res->box_word->length() != word_length_) {
+      if (word_res->box_word->length() != static_cast<unsigned>(word_length_)) {
         tprintf("Corrupted word! best_choice[len=%d] = %s, box_word[len=%d]: ",
-                word_length_, word_res->best_choice->unichar_string().string(),
+                word_length_, word_res->best_choice->unichar_string().c_str(),
                 word_res->box_word->length());
         word_res->box_word->bounding_box().print();
       }
-      ASSERT_HOST(word_res->box_word->length() == word_length_);
+      ASSERT_HOST(word_res->box_word->length() ==
+                  static_cast<unsigned>(word_length_));
     }
     word_ = nullptr;
     // We will be iterating the box_word.
@@ -606,12 +657,15 @@ void PageIterator::BeginWord(int offset) {
     word_ = word_res->word;
     ASSERT_HOST(word_->cblob_list() != nullptr);
     word_length_ = word_->cblob_list()->length();
-    if (cblob_it_ == nullptr) cblob_it_ = new C_BLOB_IT;
+    if (cblob_it_ == nullptr) {
+      cblob_it_ = new C_BLOB_IT;
+    }
     cblob_it_->set_to_list(word_->cblob_list());
   }
   for (blob_index_ = 0; blob_index_ < offset; ++blob_index_) {
-    if (cblob_it_ != nullptr)
+    if (cblob_it_ != nullptr) {
       cblob_it_->forward();
+    }
   }
 }
 
@@ -624,4 +678,4 @@ bool PageIterator::SetWordBlamerBundle(BlamerBundle *blamer_bundle) {
   }
 }
 
-}  // namespace tesseract.
+} // namespace tesseract.
