@@ -2,7 +2,6 @@
 // File:        lstm.cpp
 // Description: Long-term-short-term-memory Recurrent neural network.
 // Author:      Ray Smith
-// Created:     Wed May 01 17:43:06 PST 2013
 //
 // (C) Copyright 2013, Google Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,16 +15,21 @@
 // limitations under the License.
 ///////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_CONFIG_H
+#  include "config_auto.h"
+#endif
+
 #include "lstm.h"
 
 #ifdef _OPENMP
-#include <omp.h>
+#  include <omp.h>
 #endif
 #include <cstdio>
 #include <cstdlib>
+#include <sstream> // for std::ostringstream
 
 #if !defined(__GNUC__) && defined(_MSC_VER)
-#include <intrin.h>     // _BitScanReverse
+#  include <intrin.h> // _BitScanReverse
 #endif
 
 #include "fullyconnected.h"
@@ -35,45 +39,41 @@
 
 // Macros for openmp code if it is available, otherwise empty macros.
 #ifdef _OPENMP
-#define PARALLEL_IF_OPENMP(__num_threads)                                  \
-  PRAGMA(omp parallel if (__num_threads > 1) num_threads(__num_threads)) { \
-    PRAGMA(omp sections nowait) {                                          \
-      PRAGMA(omp section) {
-#define SECTION_IF_OPENMP \
-        } \
-    PRAGMA(omp section) \
-        {
-
-#define END_PARALLEL_IF_OPENMP \
-        } \
-      }  /* end of sections */ \
-    }  /* end of parallel section */
+#  define PARALLEL_IF_OPENMP(__num_threads)                                  \
+    PRAGMA(omp parallel if (__num_threads > 1) num_threads(__num_threads)) { \
+      PRAGMA(omp sections nowait) {                                          \
+        PRAGMA(omp section) {
+#  define SECTION_IF_OPENMP \
+    }                       \
+    PRAGMA(omp section) {
+#  define END_PARALLEL_IF_OPENMP \
+    }                            \
+    } /* end of sections */      \
+    } /* end of parallel section */
 
 // Define the portable PRAGMA macro.
-#ifdef _MSC_VER  // Different _Pragma
-#define PRAGMA(x) __pragma(x)
-#else
-#define PRAGMA(x) _Pragma(#x)
-#endif  // _MSC_VER
-
-#else  // _OPENMP
-#define PARALLEL_IF_OPENMP(__num_threads)
-#define SECTION_IF_OPENMP
-#define END_PARALLEL_IF_OPENMP
-#endif  // _OPENMP
+#  ifdef _MSC_VER // Different _Pragma
+#    define PRAGMA(x) __pragma(x)
+#  else
+#    define PRAGMA(x) _Pragma(#    x)
+#  endif // _MSC_VER
 
+#else // _OPENMP
+#  define PARALLEL_IF_OPENMP(__num_threads)
+#  define SECTION_IF_OPENMP
+#  define END_PARALLEL_IF_OPENMP
+#endif // _OPENMP
 
 namespace tesseract {
 
 // Max absolute value of state_. It is reasonably high to enable the state
 // to count things.
-const double kStateClip = 100.0;
+const TFloat kStateClip = 100.0;
 // Max absolute value of gate_errors (the gradients).
-const double kErrClip = 1.0f;
+const TFloat kErrClip = 1.0f;
 
 // Calculate ceil(log2(n)).
-static inline uint32_t ceil_log2(uint32_t n)
-{
+static inline uint32_t ceil_log2(uint32_t n) {
   // l2 = (unsigned)log2(n).
 #if defined(__GNUC__)
   // Use fast inline assembler code for gcc or clang.
@@ -83,29 +83,32 @@ static inline uint32_t ceil_log2(uint32_t n)
   unsigned long l2 = 0;
   _BitScanReverse(&l2, n);
 #else
-  if (n == 0) return UINT_MAX;
-  if (n == 1) return 0;
+  if (n == 0)
+    return UINT_MAX;
+  if (n == 1)
+    return 0;
   uint32_t val = n;
   uint32_t l2 = 0;
   while (val > 1) {
-      val >>= 1;
-      l2++;
+    val >>= 1;
+    l2++;
   }
 #endif
   // Round up if n is not a power of 2.
   return (n == (1u << l2)) ? l2 : l2 + 1;
 }
 
-LSTM::LSTM(const STRING& name, int ni, int ns, int no, bool two_dimensional,
-           NetworkType type)
-    : Network(type, name, ni, no),
-      na_(ni + ns),
-      ns_(ns),
-      nf_(0),
-      is_2d_(two_dimensional),
-      softmax_(nullptr),
-      input_width_(0) {
-  if (two_dimensional) na_ += ns_;
+LSTM::LSTM(const std::string &name, int ni, int ns, int no, bool two_dimensional, NetworkType type)
+    : Network(type, name, ni, no)
+    , na_(ni + ns)
+    , ns_(ns)
+    , nf_(0)
+    , is_2d_(two_dimensional)
+    , softmax_(nullptr)
+    , input_width_(0) {
+  if (two_dimensional) {
+    na_ += ns_;
+  }
   if (type_ == NT_LSTM || type_ == NT_LSTM_SUMMARY) {
     nf_ = 0;
     // networkbuilder ensures this is always true.
@@ -120,15 +123,21 @@ LSTM::LSTM(const STRING& name, int ni, int ns, int no, bool two_dimensional,
   na_ += nf_;
 }
 
-LSTM::~LSTM() { delete softmax_; }
+LSTM::~LSTM() {
+  delete softmax_;
+}
 
 // Returns the shape output from the network given an input shape (which may
 // be partially unknown ie zero).
-StaticShape LSTM::OutputShape(const StaticShape& input_shape) const {
+StaticShape LSTM::OutputShape(const StaticShape &input_shape) const {
   StaticShape result = input_shape;
   result.set_depth(no_);
-  if (type_ == NT_LSTM_SUMMARY) result.set_width(1);
-  if (softmax_ != nullptr) return softmax_->OutputShape(result);
+  if (type_ == NT_LSTM_SUMMARY) {
+    result.set_width(1);
+  }
+  if (softmax_ != nullptr) {
+    return softmax_->OutputShape(result);
+  }
   return result;
 }
 
@@ -137,31 +146,41 @@ StaticShape LSTM::OutputShape(const StaticShape& input_shape) const {
 void LSTM::SetEnableTraining(TrainingState state) {
   if (state == TS_RE_ENABLE) {
     // Enable only from temp disabled.
-    if (training_ == TS_TEMP_DISABLE) training_ = TS_ENABLED;
+    if (training_ == TS_TEMP_DISABLE) {
+      training_ = TS_ENABLED;
+    }
   } else if (state == TS_TEMP_DISABLE) {
     // Temp disable only from enabled.
-    if (training_ == TS_ENABLED) training_ = state;
+    if (training_ == TS_ENABLED) {
+      training_ = state;
+    }
   } else {
     if (state == TS_ENABLED && training_ != TS_ENABLED) {
       for (int w = 0; w < WT_COUNT; ++w) {
-        if (w == GFS && !Is2D()) continue;
+        if (w == GFS && !Is2D()) {
+          continue;
+        }
         gate_weights_[w].InitBackward();
       }
     }
     training_ = state;
   }
-  if (softmax_ != nullptr) softmax_->SetEnableTraining(state);
+  if (softmax_ != nullptr) {
+    softmax_->SetEnableTraining(state);
+  }
 }
 
 // Sets up the network for training. Initializes weights using weights of
 // scale `range` picked according to the random number generator `randomizer`.
-int LSTM::InitWeights(float range, TRand* randomizer) {
+int LSTM::InitWeights(float range, TRand *randomizer) {
   Network::SetRandomizer(randomizer);
   num_weights_ = 0;
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
-    num_weights_ += gate_weights_[w].InitWeightsFloat(
-        ns_, na_ + 1, TestFlag(NF_ADAM), range, randomizer);
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
+    num_weights_ +=
+        gate_weights_[w].InitWeightsFloat(ns_, na_ + 1, TestFlag(NF_ADAM), range, randomizer);
   }
   if (softmax_ != nullptr) {
     num_weights_ += softmax_->InitWeights(range, randomizer);
@@ -171,7 +190,7 @@ int LSTM::InitWeights(float range, TRand* randomizer) {
 
 // Recursively searches the network for softmaxes with old_no outputs,
 // and remaps their outputs according to code_map. See network.h for details.
-int LSTM::RemapOutputs(int old_no, const std::vector<int>& code_map) {
+int LSTM::RemapOutputs(int old_no, const std::vector<int> &code_map) {
   if (softmax_ != nullptr) {
     num_weights_ -= softmax_->num_weights();
     num_weights_ += softmax_->RemapOutputs(old_no, code_map);
@@ -182,7 +201,9 @@ int LSTM::RemapOutputs(int old_no, const std::vector<int>& code_map) {
 // Converts a float network to an int network.
 void LSTM::ConvertToInt() {
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     gate_weights_[w].ConvertToInt();
   }
   if (softmax_ != nullptr) {
@@ -193,10 +214,12 @@ void LSTM::ConvertToInt() {
 // Sets up the network for training using the given weight_range.
 void LSTM::DebugWeights() {
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
-    STRING msg = name_;
-    msg.add_str_int(" Gate weights ", w);
-    gate_weights_[w].Debug2D(msg.string());
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
+    std::ostringstream msg;
+    msg << name_ << " Gate weights " << w;
+    gate_weights_[w].Debug2D(msg.str().c_str());
   }
   if (softmax_ != nullptr) {
     softmax_->DebugWeights();
@@ -204,21 +227,33 @@ void LSTM::DebugWeights() {
 }
 
 // Writes to the given file. Returns false in case of error.
-bool LSTM::Serialize(TFile* fp) const {
-  if (!Network::Serialize(fp)) return false;
-  if (!fp->Serialize(&na_)) return false;
+bool LSTM::Serialize(TFile *fp) const {
+  if (!Network::Serialize(fp)) {
+    return false;
+  }
+  if (!fp->Serialize(&na_)) {
+    return false;
+  }
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
-    if (!gate_weights_[w].Serialize(IsTraining(), fp)) return false;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
+    if (!gate_weights_[w].Serialize(IsTraining(), fp)) {
+      return false;
+    }
+  }
+  if (softmax_ != nullptr && !softmax_->Serialize(fp)) {
+    return false;
   }
-  if (softmax_ != nullptr && !softmax_->Serialize(fp)) return false;
   return true;
 }
 
 // Reads from the given file. Returns false in case of error.
 
-bool LSTM::DeSerialize(TFile* fp) {
-  if (!fp->DeSerialize(&na_)) return false;
+bool LSTM::DeSerialize(TFile *fp) {
+  if (!fp->DeSerialize(&na_)) {
+    return false;
+  }
   if (type_ == NT_LSTM_SOFTMAX) {
     nf_ = no_;
   } else if (type_ == NT_LSTM_SOFTMAX_ENCODED) {
@@ -228,8 +263,12 @@ bool LSTM::DeSerialize(TFile* fp) {
   }
   is_2d_ = false;
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
-    if (!gate_weights_[w].DeSerialize(IsTraining(), fp)) return false;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
+    if (!gate_weights_[w].DeSerialize(IsTraining(), fp)) {
+      return false;
+    }
     if (w == CI) {
       ns_ = gate_weights_[CI].NumOutputs();
       is_2d_ = na_ - nf_ == ni_ + 2 * ns_;
@@ -237,8 +276,10 @@ bool LSTM::DeSerialize(TFile* fp) {
   }
   delete softmax_;
   if (type_ == NT_LSTM_SOFTMAX || type_ == NT_LSTM_SOFTMAX_ENCODED) {
-    softmax_ = static_cast<FullyConnected*>(Network::CreateFromFile(fp));
-    if (softmax_ == nullptr) return false;
+    softmax_ = static_cast<FullyConnected *>(Network::CreateFromFile(fp));
+    if (softmax_ == nullptr) {
+      return false;
+    }
   } else {
     softmax_ = nullptr;
   }
@@ -247,40 +288,46 @@ bool LSTM::DeSerialize(TFile* fp) {
 
 // Runs forward propagation of activations on the input line.
 // See NetworkCpp for a detailed discussion of the arguments.
-void LSTM::Forward(bool debug, const NetworkIO& input,
-                   const TransposedArray* input_transpose,
-                   NetworkScratch* scratch, NetworkIO* output) {
+void LSTM::Forward(bool debug, const NetworkIO &input, const TransposedArray *input_transpose,
+                   NetworkScratch *scratch, NetworkIO *output) {
   input_map_ = input.stride_map();
   input_width_ = input.Width();
-  if (softmax_ != nullptr)
+  if (softmax_ != nullptr) {
     output->ResizeFloat(input, no_);
-  else if (type_ == NT_LSTM_SUMMARY)
+  } else if (type_ == NT_LSTM_SUMMARY) {
     output->ResizeXTo1(input, no_);
-  else
+  } else {
     output->Resize(input, no_);
+  }
   ResizeForward(input);
   // Temporary storage of forward computation for each gate.
   NetworkScratch::FloatVec temp_lines[WT_COUNT];
-  for (int i = 0; i < WT_COUNT; ++i) temp_lines[i].Init(ns_, scratch);
+  int ro = ns_;
+  if (source_.int_mode() && IntSimdMatrix::intSimdMatrix) {
+    ro = IntSimdMatrix::intSimdMatrix->RoundOutputs(ro);
+  }
+  for (auto &temp_line : temp_lines) {
+    temp_line.Init(ns_, ro, scratch);
+  }
   // Single timestep buffers for the current/recurrent output and state.
   NetworkScratch::FloatVec curr_state, curr_output;
   curr_state.Init(ns_, scratch);
-  ZeroVector<double>(ns_, curr_state);
+  ZeroVector<TFloat>(ns_, curr_state);
   curr_output.Init(ns_, scratch);
-  ZeroVector<double>(ns_, curr_output);
+  ZeroVector<TFloat>(ns_, curr_output);
   // Rotating buffers of width buf_width allow storage of the state and output
   // for the other dimension, used only when working in true 2D mode. The width
   // is enough to hold an entire strip of the major direction.
   int buf_width = Is2D() ? input_map_.Size(FD_WIDTH) : 1;
-  GenericVector<NetworkScratch::FloatVec> states, outputs;
+  std::vector<NetworkScratch::FloatVec> states, outputs;
   if (Is2D()) {
-    states.init_to_size(buf_width, NetworkScratch::FloatVec());
-    outputs.init_to_size(buf_width, NetworkScratch::FloatVec());
+    states.resize(buf_width);
+    outputs.resize(buf_width);
     for (int i = 0; i < buf_width; ++i) {
       states[i].Init(ns_, scratch);
-      ZeroVector<double>(ns_, states[i]);
+      ZeroVector<TFloat>(ns_, states[i]);
       outputs[i].Init(ns_, scratch);
-      ZeroVector<double>(ns_, outputs[i]);
+      ZeroVector<TFloat>(ns_, outputs[i]);
     }
   }
   // Used only if a softmax LSTM.
@@ -288,10 +335,11 @@ void LSTM::Forward(bool debug, const NetworkIO& input,
   NetworkScratch::IO int_output;
   if (softmax_ != nullptr) {
     softmax_output.Init(no_, scratch);
-    ZeroVector<double>(no_, softmax_output);
+    ZeroVector<TFloat>(no_, softmax_output);
     int rounded_softmax_inputs = gate_weights_[CI].RoundInputs(ns_);
-    if (input.int_mode())
+    if (input.int_mode()) {
       int_output.Resize2d(true, 1, rounded_softmax_inputs, scratch);
+    }
     softmax_->SetupForward(input, nullptr);
   }
   NetworkScratch::FloatVec curr_input;
@@ -305,62 +353,72 @@ void LSTM::Forward(bool debug, const NetworkIO& input,
     bool valid_2d = Is2D();
     if (valid_2d) {
       StrideMap::Index dim_index(src_index);
-      if (!dim_index.AddOffset(-1, FD_HEIGHT)) valid_2d = false;
+      if (!dim_index.AddOffset(-1, FD_HEIGHT)) {
+        valid_2d = false;
+      }
     }
     // Index of the 2-D revolving buffers (outputs, states).
-    int mod_t = Modulo(t, buf_width);      // Current timestep.
+    int mod_t = Modulo(t, buf_width); // Current timestep.
     // Setup the padded input in source.
     source_.CopyTimeStepGeneral(t, 0, ni_, input, t, 0);
     if (softmax_ != nullptr) {
       source_.WriteTimeStepPart(t, ni_, nf_, softmax_output);
     }
     source_.WriteTimeStepPart(t, ni_ + nf_, ns_, curr_output);
-    if (Is2D())
+    if (Is2D()) {
       source_.WriteTimeStepPart(t, ni_ + nf_ + ns_, ns_, outputs[mod_t]);
-    if (!source_.int_mode()) source_.ReadTimeStep(t, curr_input);
+    }
+    if (!source_.int_mode()) {
+      source_.ReadTimeStep(t, curr_input);
+    }
     // Matrix multiply the inputs with the source.
     PARALLEL_IF_OPENMP(GFS)
     // It looks inefficient to create the threads on each t iteration, but the
     // alternative of putting the parallel outside the t loop, a single around
     // the t-loop and then tasks in place of the sections is a *lot* slower.
     // Cell inputs.
-    if (source_.int_mode())
+    if (source_.int_mode()) {
       gate_weights_[CI].MatrixDotVector(source_.i(t), temp_lines[CI]);
-    else
+    } else {
       gate_weights_[CI].MatrixDotVector(curr_input, temp_lines[CI]);
+    }
     FuncInplace<GFunc>(ns_, temp_lines[CI]);
 
     SECTION_IF_OPENMP
     // Input Gates.
-    if (source_.int_mode())
+    if (source_.int_mode()) {
       gate_weights_[GI].MatrixDotVector(source_.i(t), temp_lines[GI]);
-    else
+    } else {
       gate_weights_[GI].MatrixDotVector(curr_input, temp_lines[GI]);
+    }
     FuncInplace<FFunc>(ns_, temp_lines[GI]);
 
     SECTION_IF_OPENMP
     // 1-D forget gates.
-    if (source_.int_mode())
+    if (source_.int_mode()) {
       gate_weights_[GF1].MatrixDotVector(source_.i(t), temp_lines[GF1]);
-    else
+    } else {
       gate_weights_[GF1].MatrixDotVector(curr_input, temp_lines[GF1]);
+    }
     FuncInplace<FFunc>(ns_, temp_lines[GF1]);
 
     // 2-D forget gates.
     if (Is2D()) {
-      if (source_.int_mode())
+      if (source_.int_mode()) {
         gate_weights_[GFS].MatrixDotVector(source_.i(t), temp_lines[GFS]);
-      else
+      } else {
         gate_weights_[GFS].MatrixDotVector(curr_input, temp_lines[GFS]);
+      }
       FuncInplace<FFunc>(ns_, temp_lines[GFS]);
     }
 
     SECTION_IF_OPENMP
     // Output gates.
-    if (source_.int_mode())
+    if (source_.int_mode()) {
       gate_weights_[GO].MatrixDotVector(source_.i(t), temp_lines[GO]);
-    else
+    } else {
       gate_weights_[GO].MatrixDotVector(curr_input, temp_lines[GO]);
+    }
     FuncInplace<FFunc>(ns_, temp_lines[GO]);
     END_PARALLEL_IF_OPENMP
 
@@ -368,10 +426,10 @@ void LSTM::Forward(bool debug, const NetworkIO& input,
     MultiplyVectorsInPlace(ns_, temp_lines[GF1], curr_state);
     if (Is2D()) {
       // Max-pool the forget gates (in 2-d) instead of blindly adding.
-      int8_t* which_fg_col = which_fg_[t];
+      int8_t *which_fg_col = which_fg_[t];
       memset(which_fg_col, 1, ns_ * sizeof(which_fg_col[0]));
       if (valid_2d) {
-        const double* stepped_state = states[mod_t];
+        const TFloat *stepped_state = states[mod_t];
         for (int i = 0; i < ns_; ++i) {
           if (temp_lines[GF1][i] < temp_lines[GFS][i]) {
             curr_state[i] = temp_lines[GFS][i] * stepped_state[i];
@@ -382,17 +440,21 @@ void LSTM::Forward(bool debug, const NetworkIO& input,
     }
     MultiplyAccumulate(ns_, temp_lines[CI], temp_lines[GI], curr_state);
     // Clip curr_state to a sane range.
-    ClipVector<double>(ns_, -kStateClip, kStateClip, curr_state);
+    ClipVector<TFloat>(ns_, -kStateClip, kStateClip, curr_state);
     if (IsTraining()) {
       // Save the gate node values.
       node_values_[CI].WriteTimeStep(t, temp_lines[CI]);
       node_values_[GI].WriteTimeStep(t, temp_lines[GI]);
       node_values_[GF1].WriteTimeStep(t, temp_lines[GF1]);
       node_values_[GO].WriteTimeStep(t, temp_lines[GO]);
-      if (Is2D()) node_values_[GFS].WriteTimeStep(t, temp_lines[GFS]);
+      if (Is2D()) {
+        node_values_[GFS].WriteTimeStep(t, temp_lines[GFS]);
+      }
     }
     FuncMultiply<HFunc>(curr_state, temp_lines[GO], ns_, curr_output);
-    if (IsTraining()) state_.WriteTimeStep(t, curr_state);
+    if (IsTraining()) {
+      state_.WriteTimeStep(t, curr_state);
+    }
     if (softmax_ != nullptr) {
       if (input.int_mode()) {
         int_output->WriteTimeStepPart(0, 0, ns_, curr_output);
@@ -421,27 +483,34 @@ void LSTM::Forward(bool debug, const NetworkIO& input,
     // Always zero the states at the end of every row, but only for the major
     // direction. The 2-D state remains intact.
     if (src_index.IsLast(FD_WIDTH)) {
-      ZeroVector<double>(ns_, curr_state);
-      ZeroVector<double>(ns_, curr_output);
+      ZeroVector<TFloat>(ns_, curr_state);
+      ZeroVector<TFloat>(ns_, curr_output);
     }
   } while (src_index.Increment());
 #if DEBUG_DETAIL > 0
-  tprintf("Source:%s\n", name_.string());
+  tprintf("Source:%s\n", name_.c_str());
   source_.Print(10);
-  tprintf("State:%s\n", name_.string());
+  tprintf("State:%s\n", name_.c_str());
   state_.Print(10);
-  tprintf("Output:%s\n", name_.string());
+  tprintf("Output:%s\n", name_.c_str());
   output->Print(10);
 #endif
-  if (debug) DisplayForward(*output);
+#ifndef GRAPHICS_DISABLED
+  if (debug) {
+    DisplayForward(*output);
+  }
+#endif
 }
 
 // Runs backward propagation of errors on the deltas line.
 // See NetworkCpp for a detailed discussion of the arguments.
-bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
-                    NetworkScratch* scratch,
-                    NetworkIO* back_deltas) {
-  if (debug) DisplayBackward(fwd_deltas);
+bool LSTM::Backward(bool debug, const NetworkIO &fwd_deltas, NetworkScratch *scratch,
+                    NetworkIO *back_deltas) {
+#ifndef GRAPHICS_DISABLED
+  if (debug) {
+    DisplayBackward(fwd_deltas);
+  }
+#endif
   back_deltas->ResizeToMap(fwd_deltas.int_mode(), input_map_, ni_);
   // ======Scratch space.======
   // Output errors from deltas with recurrence from sourceerr.
@@ -451,34 +520,37 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
   NetworkScratch::FloatVec curr_stateerr, curr_sourceerr;
   curr_stateerr.Init(ns_, scratch);
   curr_sourceerr.Init(na_, scratch);
-  ZeroVector<double>(ns_, curr_stateerr);
-  ZeroVector<double>(na_, curr_sourceerr);
+  ZeroVector<TFloat>(ns_, curr_stateerr);
+  ZeroVector<TFloat>(na_, curr_sourceerr);
   // Errors in the gates.
   NetworkScratch::FloatVec gate_errors[WT_COUNT];
-  for (int g = 0; g < WT_COUNT; ++g) gate_errors[g].Init(ns_, scratch);
+  for (auto &gate_error : gate_errors) {
+    gate_error.Init(ns_, scratch);
+  }
   // Rotating buffers of width buf_width allow storage of the recurrent time-
   // steps used only for true 2-D. Stores one full strip of the major direction.
   int buf_width = Is2D() ? input_map_.Size(FD_WIDTH) : 1;
-  GenericVector<NetworkScratch::FloatVec> stateerr, sourceerr;
+  std::vector<NetworkScratch::FloatVec> stateerr, sourceerr;
   if (Is2D()) {
-    stateerr.init_to_size(buf_width, NetworkScratch::FloatVec());
-    sourceerr.init_to_size(buf_width, NetworkScratch::FloatVec());
+    stateerr.resize(buf_width);
+    sourceerr.resize(buf_width);
     for (int t = 0; t < buf_width; ++t) {
       stateerr[t].Init(ns_, scratch);
       sourceerr[t].Init(na_, scratch);
-      ZeroVector<double>(ns_, stateerr[t]);
-      ZeroVector<double>(na_, sourceerr[t]);
+      ZeroVector<TFloat>(ns_, stateerr[t]);
+      ZeroVector<TFloat>(na_, sourceerr[t]);
     }
   }
   // Parallel-generated sourceerr from each of the gates.
   NetworkScratch::FloatVec sourceerr_temps[WT_COUNT];
-  for (int w = 0; w < WT_COUNT; ++w)
-    sourceerr_temps[w].Init(na_, scratch);
+  for (auto &sourceerr_temp : sourceerr_temps) {
+    sourceerr_temp.Init(na_, scratch);
+  }
   int width = input_width_;
   // Transposed gate errors stored over all timesteps for sum outer.
   NetworkScratch::GradientStore gate_errors_t[WT_COUNT];
-  for (int w = 0; w < WT_COUNT; ++w) {
-    gate_errors_t[w].Init(ns_, width, scratch);
+  for (auto &w : gate_errors_t) {
+    w.Init(ns_, width, scratch);
   }
   // Used only if softmax_ != nullptr.
   NetworkScratch::FloatVec softmax_errors;
@@ -487,9 +559,9 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
     softmax_errors.Init(no_, scratch);
     softmax_errors_t.Init(no_, width, scratch);
   }
-  double state_clip = Is2D() ? 9.0 : 4.0;
+  TFloat state_clip = Is2D() ? 9.0 : 4.0;
 #if DEBUG_DETAIL > 1
-  tprintf("fwd_deltas:%s\n", name_.string());
+  tprintf("fwd_deltas:%s\n", name_.c_str());
   fwd_deltas.Print(10);
 #endif
   StrideMap::Index dest_index(input_map_);
@@ -507,19 +579,23 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
     if (Is2D()) {
       if (dest_index.index(FD_HEIGHT) > 0) {
         StrideMap::Index up_index(dest_index);
-        if (up_index.AddOffset(-1, FD_HEIGHT)) up_pos = up_index.t();
+        if (up_index.AddOffset(-1, FD_HEIGHT)) {
+          up_pos = up_index.t();
+        }
       }
       if (!dest_index.IsLast(FD_HEIGHT)) {
         StrideMap::Index down_index(dest_index);
-        if (down_index.AddOffset(1, FD_HEIGHT)) down_pos = down_index.t();
+        if (down_index.AddOffset(1, FD_HEIGHT)) {
+          down_pos = down_index.t();
+        }
       }
     }
     // Index of the 2-D revolving buffers (sourceerr, stateerr).
-    int mod_t = Modulo(t, buf_width);      // Current timestep.
+    int mod_t = Modulo(t, buf_width); // Current timestep.
     // Zero the state in the major direction only at the end of every row.
     if (at_last_x) {
-      ZeroVector<double>(na_, curr_sourceerr);
-      ZeroVector<double>(ns_, curr_stateerr);
+      ZeroVector<TFloat>(na_, curr_sourceerr);
+      ZeroVector<TFloat>(ns_, curr_stateerr);
     }
     // Setup the outputerr.
     if (type_ == NT_LSTM_SUMMARY) {
@@ -527,32 +603,35 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
         fwd_deltas.ReadTimeStep(src_index.t(), outputerr);
         src_index.Decrement();
       } else {
-        ZeroVector<double>(ns_, outputerr);
+        ZeroVector<TFloat>(ns_, outputerr);
       }
     } else if (softmax_ == nullptr) {
       fwd_deltas.ReadTimeStep(t, outputerr);
     } else {
-      softmax_->BackwardTimeStep(fwd_deltas, t, softmax_errors,
-                                 softmax_errors_t.get(), outputerr);
+      softmax_->BackwardTimeStep(fwd_deltas, t, softmax_errors, softmax_errors_t.get(), outputerr);
     }
-    if (!at_last_x)
+    if (!at_last_x) {
       AccumulateVector(ns_, curr_sourceerr + ni_ + nf_, outputerr);
-    if (down_pos >= 0)
+    }
+    if (down_pos >= 0) {
       AccumulateVector(ns_, sourceerr[mod_t] + ni_ + nf_ + ns_, outputerr);
+    }
     // Apply the 1-d forget gates.
     if (!at_last_x) {
-      const float* next_node_gf1 = node_values_[GF1].f(t + 1);
+      const float *next_node_gf1 = node_values_[GF1].f(t + 1);
       for (int i = 0; i < ns_; ++i) {
         curr_stateerr[i] *= next_node_gf1[i];
       }
     }
     if (Is2D() && t + 1 < width) {
       for (int i = 0; i < ns_; ++i) {
-        if (which_fg_[t + 1][i] != 1) curr_stateerr[i] = 0.0;
+        if (which_fg_[t + 1][i] != 1) {
+          curr_stateerr[i] = 0.0;
+        }
       }
       if (down_pos >= 0) {
-        const float* right_node_gfs = node_values_[GFS].f(down_pos);
-        const double* right_stateerr = stateerr[mod_t];
+        const float *right_node_gfs = node_values_[GFS].f(down_pos);
+        const TFloat *right_stateerr = stateerr[mod_t];
         for (int i = 0; i < ns_; ++i) {
           if (which_fg_[down_pos][i] == 2) {
             curr_stateerr[i] += right_stateerr[i] * right_node_gfs[i];
@@ -560,16 +639,14 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
         }
       }
     }
-    state_.FuncMultiply3Add<HPrime>(node_values_[GO], t, outputerr,
-                                    curr_stateerr);
+    state_.FuncMultiply3Add<HPrime>(node_values_[GO], t, outputerr, curr_stateerr);
     // Clip stateerr_ to a sane range.
-    ClipVector<double>(ns_, -state_clip, state_clip, curr_stateerr);
+    ClipVector<TFloat>(ns_, -state_clip, state_clip, curr_stateerr);
 #if DEBUG_DETAIL > 1
     if (t + 10 > width) {
       tprintf("t=%d, stateerr=", t);
       for (int i = 0; i < ns_; ++i)
-        tprintf(" %g,%g,%g", curr_stateerr[i], outputerr[i],
-                curr_sourceerr[ni_ + nf_ + i]);
+        tprintf(" %g,%g,%g", curr_stateerr[i], outputerr[i], curr_sourceerr[ni_ + nf_ + i]);
       tprintf("\n");
     }
 #endif
@@ -577,16 +654,14 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
     PARALLEL_IF_OPENMP(GFS)
 
     // Cell inputs.
-    node_values_[CI].FuncMultiply3<GPrime>(t, node_values_[GI], t,
-                                           curr_stateerr, gate_errors[CI]);
+    node_values_[CI].FuncMultiply3<GPrime>(t, node_values_[GI], t, curr_stateerr, gate_errors[CI]);
     ClipVector(ns_, -kErrClip, kErrClip, gate_errors[CI].get());
     gate_weights_[CI].VectorDotMatrix(gate_errors[CI], sourceerr_temps[CI]);
     gate_errors_t[CI].get()->WriteStrided(t, gate_errors[CI]);
 
     SECTION_IF_OPENMP
     // Input Gates.
-    node_values_[GI].FuncMultiply3<FPrime>(t, node_values_[CI], t,
-                                           curr_stateerr, gate_errors[GI]);
+    node_values_[GI].FuncMultiply3<FPrime>(t, node_values_[CI], t, curr_stateerr, gate_errors[GI]);
     ClipVector(ns_, -kErrClip, kErrClip, gate_errors[GI].get());
     gate_weights_[GI].VectorDotMatrix(gate_errors[GI], sourceerr_temps[GI]);
     gate_errors_t[GI].get()->WriteStrided(t, gate_errors[GI]);
@@ -594,11 +669,9 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
     SECTION_IF_OPENMP
     // 1-D forget Gates.
     if (t > 0) {
-      node_values_[GF1].FuncMultiply3<FPrime>(t, state_, t - 1, curr_stateerr,
-                                              gate_errors[GF1]);
+      node_values_[GF1].FuncMultiply3<FPrime>(t, state_, t - 1, curr_stateerr, gate_errors[GF1]);
       ClipVector(ns_, -kErrClip, kErrClip, gate_errors[GF1].get());
-      gate_weights_[GF1].VectorDotMatrix(gate_errors[GF1],
-                                         sourceerr_temps[GF1]);
+      gate_weights_[GF1].VectorDotMatrix(gate_errors[GF1], sourceerr_temps[GF1]);
     } else {
       memset(gate_errors[GF1], 0, ns_ * sizeof(gate_errors[GF1][0]));
       memset(sourceerr_temps[GF1], 0, na_ * sizeof(*sourceerr_temps[GF1]));
@@ -607,29 +680,27 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
 
     // 2-D forget Gates.
     if (up_pos >= 0) {
-      node_values_[GFS].FuncMultiply3<FPrime>(t, state_, up_pos, curr_stateerr,
-                                              gate_errors[GFS]);
+      node_values_[GFS].FuncMultiply3<FPrime>(t, state_, up_pos, curr_stateerr, gate_errors[GFS]);
       ClipVector(ns_, -kErrClip, kErrClip, gate_errors[GFS].get());
-      gate_weights_[GFS].VectorDotMatrix(gate_errors[GFS],
-                                         sourceerr_temps[GFS]);
+      gate_weights_[GFS].VectorDotMatrix(gate_errors[GFS], sourceerr_temps[GFS]);
     } else {
       memset(gate_errors[GFS], 0, ns_ * sizeof(gate_errors[GFS][0]));
       memset(sourceerr_temps[GFS], 0, na_ * sizeof(*sourceerr_temps[GFS]));
     }
-    if (Is2D()) gate_errors_t[GFS].get()->WriteStrided(t, gate_errors[GFS]);
+    if (Is2D()) {
+      gate_errors_t[GFS].get()->WriteStrided(t, gate_errors[GFS]);
+    }
 
     SECTION_IF_OPENMP
     // Output gates.
-    state_.Func2Multiply3<HFunc, FPrime>(node_values_[GO], t, outputerr,
-                                         gate_errors[GO]);
+    state_.Func2Multiply3<HFunc, FPrime>(node_values_[GO], t, outputerr, gate_errors[GO]);
     ClipVector(ns_, -kErrClip, kErrClip, gate_errors[GO].get());
     gate_weights_[GO].VectorDotMatrix(gate_errors[GO], sourceerr_temps[GO]);
     gate_errors_t[GO].get()->WriteStrided(t, gate_errors[GO]);
     END_PARALLEL_IF_OPENMP
 
-    SumVectors(na_, sourceerr_temps[CI], sourceerr_temps[GI],
-               sourceerr_temps[GF1], sourceerr_temps[GO], sourceerr_temps[GFS],
-               curr_sourceerr);
+    SumVectors(na_, sourceerr_temps[CI], sourceerr_temps[GI], sourceerr_temps[GF1],
+               sourceerr_temps[GO], sourceerr_temps[GFS], curr_sourceerr);
     back_deltas->WriteTimeStep(t, curr_sourceerr);
     // Save states for use by the 2nd dimension only if needed.
     if (Is2D()) {
@@ -639,7 +710,7 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
   } while (dest_index.Decrement());
 #if DEBUG_DETAIL > 2
   for (int w = 0; w < WT_COUNT; ++w) {
-    tprintf("%s gate errors[%d]\n", name_.string(), w);
+    tprintf("%s gate errors[%d]\n", name_.c_str(), w);
     gate_errors_t[w].get()->PrintUnTransposed(10);
   }
 #endif
@@ -650,10 +721,12 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
   state_t.Init(ns_, width, scratch);
   state_.Transpose(state_t.get());
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(GFS) if (!Is2D())
+#  pragma omp parallel for num_threads(GFS) if (!Is2D())
 #endif
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     gate_weights_[w].SumOuterTransposed(*gate_errors_t[w], *source_t, false);
   }
   if (softmax_ != nullptr) {
@@ -664,13 +737,14 @@ bool LSTM::Backward(bool debug, const NetworkIO& fwd_deltas,
 
 // Updates the weights using the given learning rate, momentum and adam_beta.
 // num_samples is used in the adam computation iff use_adam_ is true.
-void LSTM::Update(float learning_rate, float momentum, float adam_beta,
-                  int num_samples) {
+void LSTM::Update(float learning_rate, float momentum, float adam_beta, int num_samples) {
 #if DEBUG_DETAIL > 3
   PrintW();
 #endif
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     gate_weights_[w].Update(learning_rate, momentum, adam_beta, num_samples);
   }
   if (softmax_ != nullptr) {
@@ -684,12 +758,13 @@ void LSTM::Update(float learning_rate, float momentum, float adam_beta,
 // Sums the products of weight updates in *this and other, splitting into
 // positive (same direction) in *same and negative (different direction) in
 // *changed.
-void LSTM::CountAlternators(const Network& other, double* same,
-                            double* changed) const {
+void LSTM::CountAlternators(const Network &other, TFloat *same, TFloat *changed) const {
   ASSERT_HOST(other.type() == type_);
-  const LSTM* lstm = static_cast<const LSTM*>(&other);
+  const LSTM *lstm = static_cast<const LSTM *>(&other);
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     gate_weights_[w].CountAlternators(lstm->gate_weights_[w], same, changed);
   }
   if (softmax_ != nullptr) {
@@ -697,71 +772,86 @@ void LSTM::CountAlternators(const Network& other, double* same,
   }
 }
 
+#if DEBUG_DETAIL > 3
+
 // Prints the weights for debug purposes.
 void LSTM::PrintW() {
-  tprintf("Weight state:%s\n", name_.string());
+  tprintf("Weight state:%s\n", name_.c_str());
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     tprintf("Gate %d, inputs\n", w);
     for (int i = 0; i < ni_; ++i) {
       tprintf("Row %d:", i);
-      for (int s = 0; s < ns_; ++s)
+      for (int s = 0; s < ns_; ++s) {
         tprintf(" %g", gate_weights_[w].GetWeights(s)[i]);
+      }
       tprintf("\n");
     }
     tprintf("Gate %d, outputs\n", w);
     for (int i = ni_; i < ni_ + ns_; ++i) {
       tprintf("Row %d:", i - ni_);
-      for (int s = 0; s < ns_; ++s)
+      for (int s = 0; s < ns_; ++s) {
         tprintf(" %g", gate_weights_[w].GetWeights(s)[i]);
+      }
       tprintf("\n");
     }
     tprintf("Gate %d, bias\n", w);
-    for (int s = 0; s < ns_; ++s)
+    for (int s = 0; s < ns_; ++s) {
       tprintf(" %g", gate_weights_[w].GetWeights(s)[na_]);
+    }
     tprintf("\n");
   }
 }
 
 // Prints the weight deltas for debug purposes.
 void LSTM::PrintDW() {
-  tprintf("Delta state:%s\n", name_.string());
+  tprintf("Delta state:%s\n", name_.c_str());
   for (int w = 0; w < WT_COUNT; ++w) {
-    if (w == GFS && !Is2D()) continue;
+    if (w == GFS && !Is2D()) {
+      continue;
+    }
     tprintf("Gate %d, inputs\n", w);
     for (int i = 0; i < ni_; ++i) {
       tprintf("Row %d:", i);
-      for (int s = 0; s < ns_; ++s)
+      for (int s = 0; s < ns_; ++s) {
         tprintf(" %g", gate_weights_[w].GetDW(s, i));
+      }
       tprintf("\n");
     }
     tprintf("Gate %d, outputs\n", w);
     for (int i = ni_; i < ni_ + ns_; ++i) {
       tprintf("Row %d:", i - ni_);
-      for (int s = 0; s < ns_; ++s)
+      for (int s = 0; s < ns_; ++s) {
         tprintf(" %g", gate_weights_[w].GetDW(s, i));
+      }
       tprintf("\n");
     }
     tprintf("Gate %d, bias\n", w);
-    for (int s = 0; s < ns_; ++s)
+    for (int s = 0; s < ns_; ++s) {
       tprintf(" %g", gate_weights_[w].GetDW(s, na_));
+    }
     tprintf("\n");
   }
 }
 
+#endif
+
 // Resizes forward data to cope with an input image of the given width.
-void LSTM::ResizeForward(const NetworkIO& input) {
+void LSTM::ResizeForward(const NetworkIO &input) {
   int rounded_inputs = gate_weights_[CI].RoundInputs(na_);
   source_.Resize(input, rounded_inputs);
   which_fg_.ResizeNoInit(input.Width(), ns_);
   if (IsTraining()) {
     state_.ResizeFloat(input, ns_);
     for (int w = 0; w < WT_COUNT; ++w) {
-      if (w == GFS && !Is2D()) continue;
+      if (w == GFS && !Is2D()) {
+        continue;
+      }
       node_values_[w].ResizeFloat(input, ns_);
     }
   }
 }
 
-
-}  // namespace tesseract.
+} // namespace tesseract.
